<?php

namespace Drupal\eav_field\Form;

use Drupal\Core\Field\FieldException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eav_field\Entity\EavAttributeInterface;
use Drupal\field\FieldStorageConfigInterface;
use Drupal\field_ui\Form\FieldConfigEditForm;

class EavAttributeValueFieldSettingsForm extends FieldConfigEditForm {

  /**
   * {@inheritDoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    $attribute = $form_state->get('attribute') /** @var EavAttributeInterface $attribute */;

    $form['#title'] = $this->t('Field settings of attribute "@attribute"', ['@attribute' => $attribute->getAdministrativeLabel()]);
    $form['label']['#access'] = FALSE;

    // Remove entity reference settings validation, because EntityReferenceItem::fieldSettingsFormValidate() uses
    // FieldConfig::getFieldStorageDefinition() which it does support only configurable fields.
    // @TODO Find a better solution (maybe convert "value" base field to configurable field?).
    if ($attribute->getValueType() == 'entity_reference') {
      unset($form['settings']['#element_validate']);
    }

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Catch FieldException, because FieldConfigBase::setDefaultValue() uses FieldConfig::getFieldStorageDefinition()
    // which it does support only configurable fields.
    // @TODO Find a better solution
    try {
      parent::submitForm($form, $form_state);
    }
    catch (FieldException $exception) {}
  }

  /**
   * {@inheritDoc}
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $attribute = $form_state->get('attribute') /** @var EavAttributeInterface $attribute */;

    $field_config = $this->entity; /** @var FieldStorageConfigInterface $field_config */
    $field_config_array = array_intersect_key($field_config->toArray(), array_flip([
      'settings',
      'default_value',
      'description',
      'required',
    ]));

    $attribute->set('value_config', $field_config_array);
    $attribute->save();

    // Redirect to next step
    if ($this->getRequest()->query->get('operation') == 'add') {
      $form_state->setRedirect('entity.eav_attribute.value_widget_form', ['eav_attribute' => $attribute->id()], ['query' => ['operation' => 'add']]);
    }
    else {
      $this->messenger()->addMessage($this->t('Field settings saved.'));
    }
  }

}
